#include <iostream>
#include <string>
#include "fdstream.hh"

extern "C" {
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <fcntl.h>
}

void copyfile (
  const char * nameIn,
  const char * nameOut
) {
  // Copy the file "nameIn" onto the file "nameOut".  NULL pointers or
  // empty strings mean the standard input or output streams.

  int fdIn;

  if (nameIn != 0   &&   *nameIn != '\0') {
    fdIn = open(nameIn, O_RDONLY);
  } else {
    fdIn = STDIN_FILENO;
  }

  if (fdIn == -1) {
    std::cerr << "Couldn't open input file " << nameIn << '\n';
  } else {
    int fdOut;

    if (nameOut != 0   &&   *nameOut != '\0') {
      fdOut = open(nameOut, O_WRONLY | O_CREAT | O_TRUNC,
                   S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
    } else {
      fdOut = STDOUT_FILENO;
    }

    if (fdOut == -1) {
      std::cerr << "Couldn't open output file " << nameOut << '\n';
    } else {
      // Copy all standard input to standard output.  The local group
      // between braces guarantees that the fdostream destructor will
      // be called before closing the fdOut descriptor (as an
      // alternate option, we could call out.flush() before closing
      // fdOut).

      {
        fdistream in(fdIn);
        fdostream out(fdOut);

        out << in.rdbuf();
      }
      close(fdOut);
    }
    close(fdIn);
  }
}

int main(
  int    argc,
  char * argv[]
) {
  // Copies the file given as first argument to the file given as
  // second argument.  Missing arguments or empty strings correspond
  // to the standard input and output streams.

  const char * nameIn  = argc > 0 ? argv[1] : 0;
  const char * nameOut = argc > 1 ? argv[2] : 0;
  copyfile(nameIn, nameOut);
  return 0;
}
